/* global
    jQuery,
    pro6pp_auth_key */

// Create closure to keep namespace clean and hide implementation.
(function($) {
  'use strict';

  var NL_FOURPP_REGEX = /[0-9]{4,4}/;
  // Trigger on '5408xb' and on '5408 XB'
  var NL_SIXPP_REGEX = /[0-9]{4,4}\s?[a-zA-Z]{2,2}/;

  $.fn.applyDistance = function(options) {
    this.apiUrl = 'https://api.pro6pp.nl/v1';
    /*jshint camelcase: false */
    this.pro6ppAuthKey = (options && options.pro6ppAuthKey) || pro6pp_auth_key;
    /*jshint camelcase: true */
    this.algorithm = (options && options.algorithm) || 'straight';
    this.country = (options && options.country) || 'nl';

    var self = this;

    this.fromPostcode = this.find('[data-binding=from-postcode]');
    this.toPostcode = this.find('[data-binding="to-postcode"]');
    this.distance = this.find('[data-binding="distance"]');
    this.duration = this.find('[data-binding="duration"]');
    this.message = this.find('[data-binding="message"]');

    this.fromPostcode.change(function() {
      distance(self);
    });
    this.toPostcode.change(function() {
      distance(self);
    });
  };

  var pro6ppCache = {};
  function pro6ppCachedGet(self, url, params, callback) {
    var key = url + $.param(params);
    if (pro6ppCache.hasOwnProperty(key)) {
      if (typeof callback !== 'undefined') {
        callback(self, pro6ppCache[key]);
      }
    } else {
      $.getJSON(url + "?callback=?", params, function(data) {
        pro6ppCache[key] = data;
        if (typeof callback !== 'undefined') {
          callback(self, data);
        }
      });
    }
  }

  function distance(self) {
    self.message.hide().empty();

    var from = self.fromPostcode.val();
    var to = self.toPostcode.val();
    if ((!(NL_FOURPP_REGEX.test(from) && NL_FOURPP_REGEX.test(to))) ||
        (!(NL_SIXPP_REGEX.test(from) && NL_SIXPP_REGEX.test(to)))) {
      return;
    }

    var algorithm = self.algorithm;
    // Get algorithm state by checking bootstrap-style checkboxes (label) as
    // used on the Pro6PP demo website. Actual users of the SDK should just
    // use the initialisation option.
    if (self.find('[data-binding="algorithm-road"].active').length > 0) {
      algorithm = 'road';
    } else if (
        self.find('[data-binding="algorithm-straight"].active').length > 0) {
      algorithm = 'straight';
    }

    /*jshint camelcase: false */
    var params = {
      auth_key: self.pro6ppAuthKey,
      algorithm: algorithm
    };

    var country = self.country;
    // Get country state by checking bootstrap-style checkboxes (label) as
    // used on the Pro6PP demo website. Actual users of the SDK should just
    // use the initialisation option.
    if (self.find('[data-binding="country-nl"].active').length > 0) {
      country = 'nl';
    } else if (
        self.find('[data-binding="country-be"].active').length > 0) {
      country = 'be';
    }

    if (country === 'nl') {
      if (from.length > 4) {
        params.from_nl_sixpp = from;
        params.to_nl_sixpp = to;
      } else {
        params.from_nl_fourpp = from;
        params.to_nl_fourpp = to;
      }
    } else if (country === 'be') {
      params.from_be_fourpp = from;
      params.to_be_fourpp = to;
    }

    /*jshint camelcase: true */
    var url = self.apiUrl + '/distance';
    pro6ppCachedGet(self, url, params, fillin);
  }

  function fillin(self, json) {
    if (json.status === 'ok') {
      self.distance.val(json.results.distance);
      if (json.results.duration) {
        self.duration.val(json.results.duration);
      }
    } else {
      self.message.text(json.error.message).show();
    }
  }
})(jQuery);
